/* NVClock 0.8 - Linux overclocker for NVIDIA cards
 * 
 * Copyright(C) 2001-2004 Roderick Colenbrander
 *
 * site: http://NVClock.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <unistd.h>
#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include <libasus/libasus.h>
#include <libasus/f75375.h>

#include "main.h"

static struct option long_options[] = {
  { "mode", 1, 0, 'm' },
  { "speed", 1, 0, 's' },
  { "help", 0, 0, 'h' },
  { "query", 0, 0, 'q' },
  { "temp", 1, 0, 't' },
  { "speeds", 1, 0, 'r' }, // Random short letter
  { "debug", 0, 0, 'd' },
  { 0, 0, 0, 0 }
};

int debug = 0;

int isnumber(const char* string) {
  char* temp;
  int i;
  
  for (i=0; i<strlen(string); i++) {
    if (!isdigit(string[i]))
      return 1;
  }
  return 0;
}

int usage() {
  printf("AsusFan v%s (libasus v%s)\n\n", ASUSFAN_VERSION, LIBASUS_VERSION);
  printf("Fan control for Asus Nvidia-based graphics cards.\n\n");
  printf("Usage ./asusfan --query\n");
  printf("Usage ./asusfan --mode=0 --speed=x\n");
  printf("Usage ./asusfan --mode=1 [--temp=x:x:x:x] [--speeds=x:x:x:x:x]\n");
  printf("Usage ./asusfan --mode=2 --speed=x\n\n");
  printf("\t-m x, --mode=x\t0 for fan speed mode, 1 for temperature mode,\n"
         "\t              \t2 for manual mode\n");
  printf("\t-s x, --speed=x\tSpeed in RPM (mode 0) or percent (mode 2)\n");
  printf("\t               \tWARNING: Turning the fan of your graphics\n");
  printf("\t               \tcard off or running it at low speeds can\n");
  printf("\t               \tcause permanent damage to your hardware.\n");
  printf("\t-q, --query\tShow card vitals\n");
  printf("\t-h, --help\tThis screen\n\n");
  printf("\t-t x:x:x:x, --temp=x:x:x:x\n");
  printf("\t\tFour temperature boundaries for temperature controlled mode\n");
  printf("\t-r x:x:x:x:, --speeds=x:x:x:x:x\n");
  printf("\t\tFive speed values, Fastest, Fast, Medium, Slow and Slowest.\n");
  printf("\t\tGiven in RPM and in the above order.\n\n");
  printf("Mode 1 defaults to:\n");
  printf("\tFastest: 3500RPM\tBoundary 1:\t90C\n");
  printf("\tFast:\t 3000RPM\tBoundary 2:\t85C\n");
  printf("\tMedium:\t 2500RPM\tBoundary 3:\t80C\n");
  printf("\tSlow:\t 2000RPM\tBoundary 4:\t75C\n");
  printf("\tSlowest: 1500RPM\n");
  return 1;
}

int main(int argc, char *argv[])
{
  libasus_card card;
  I2CDevPtr i2cdev;
  int opt, result;

  fan_vtemp new_speed;

  fan_vtemp speed = {
     { 90, 85, 80, 75 },
     { 3500, 3000, 2500, 2000, 1500 }
  };

  int opt_speed = -1, opt_mode = -1, opt_query = 0;

  if (argc == 1) {
    return usage();
  }

  while ((opt = getopt_long(argc, argv, "m:s:hqtrd", long_options, NULL)) != -1) {
    switch (opt) {
      case 'm':
	if (isnumber(optarg)) {
	  printf("Error: Illegal value for --mode: %s\n", optarg);
	  return 1;
	}
	opt_mode = strtol(optarg, (char**)NULL, 10);
	break;
      case 's':
	if (isnumber(optarg)) {
	  printf("Error: Illegal value for --speed: %s\n", optarg);
	  return 1;
	}
	opt_speed = strtol(optarg, (char**)NULL, 10);
	break;
      case 'q':
        opt_query = 1;
	break;
      case 'h':
	return usage();
	break;
      case '?':
	break;
      case 'r':
	if (sscanf(optarg, "%d:%d:%d:%d:%d", 
	  &speed.speed[0],
	  &speed.speed[1],
	  &speed.speed[2],
	  &speed.speed[3],
	  &speed.speed[4]) < 5) {
	    printf("Error: Illegal value for --speeds: %s\n", optarg);
	    return 1;
	}
	break;
      case 't':
	if (sscanf(optarg, "%d:%d:%d:%d",
	  &speed.temp[0],
	  &speed.temp[1],
	  &speed.temp[2],
	  &speed.temp[3]) < 4) {
	    printf("Error: Illegal value for --temp: %s\n", optarg);
	    return 1;
	}
	break;
      case 'd':
        debug = 1;
        break;
      default:
	return 1;
    }
  }

  if (libasus_init(debug) == false) {
    printf("ERROR: Could not initialise NVidia card\n");
    return 1;
  }

  if ((result = libasus_detect(&i2cdev)) == false) {
    printf("ERROR: Could not detect a suitable monitoring chip\n");
    return 1;
  }

  libasus_create_card(&card, i2cdev, result);
  
  if (opt_query) return query(card);

  if ((opt_mode == 0) && (opt_speed > -1)) {
    libasus_set_gpu_fanspeed(card, opt_speed);
    libasus_set_gpu_fanmode(card, MODE_SPEED);
  } else if ((opt_mode == 1)) {
    libasus_set_gpu_tempctl(card, speed);
    libasus_set_gpu_fanmode(card, MODE_TEMP);
  } else if ((opt_mode == 2)) {
    libasus_set_gpu_fanmode(card, MODE_PWM);
    libasus_set_gpu_pwmspeed(card, opt_speed > -1 ? opt_speed : 100);
  } else {
    printf("Error: Illegal Parameter Values\n");
  }

  return 0;
}

int query(libasus_card card) {

  fan_vtemp temp_fan;

  printf("GPU Temp:\t%d Degrees\n", libasus_get_gpu_temp(card));
  printf("RAM Temp:\t%d Degrees\n", libasus_get_ram_temp(card));
  printf("GPU Fan Speed:\t%d RPM\n", libasus_get_gpu_fanspeed(card));
  printf("Fan Mode:\t");

  switch (libasus_get_gpu_fanmode(card)>>4) {
    case 0:
      printf("Fan Speed Mode\n");
      break;
    case 1:
      printf("Temperature Mode\n\n");
      libasus_get_gpu_tempctl(card, &temp_fan);
      printf("Current Settings:\n");
      printf("\tFastest: %dRPM\tBoundary 1:\t%dC\n",
	temp_fan.speed[0], temp_fan.temp[0]);
      printf("\tFast:\t %dRPM\tBoundary 2:\t%dC\n",
	temp_fan.speed[1], temp_fan.temp[1]);
      printf("\tMedium:\t %dRPM\tBoundary 3:\t%dC\n",
	temp_fan.speed[2], temp_fan.temp[2]);
      printf("\tSlow:\t %dRPM\tBoundary 4:\t%dC\n",
	temp_fan.speed[3], temp_fan.temp[3]);
      printf("\tSlowest: %dRPM\n",
	temp_fan.speed[4]);
      break;
    case 2:
      printf("Manual Mode\n\n");
      printf("Current Fan Speed: %d%%\n",libasus_get_gpu_pwmspeed(card));
      break;
  }
  return 0;
}
